//Enable the use of CE WebApi
(function (webapi, $) {
    function safeAjax(ajaxOptions) {
        var deferredAjax = $.Deferred();
        shell.getTokenDeferred().done(function (token) {
            if (!ajaxOptions.headers) {
                $.extend(ajaxOptions, {
                    headers: {
                        "__RequestVerificationToken": token
                    }
                });
            } else {
                ajaxOptions.headers["__RequestVerificationToken"] = token;
            }
            $.ajax(ajaxOptions)
                .done(function (data, textStatus, jqXHR) {
                    validateLoginSession(data, textStatus, jqXHR, deferredAjax.resolve);
                }).fail(deferredAjax.reject);
        }).fail(function () {
            deferredAjax.rejectWith(this, arguments);
        });

        return deferredAjax.promise();
    }
    webapi.safeAjax = safeAjax;
})(window.webapi = window.webapi || {}, jQuery);

//Return val() as Number.
$.fn.nval = function () {
    return Number(this.val())
};
//Check if element exists
$.fn.exists = $.fn.exists || function () {
    return !!(this.length && (this[0] instanceof HTMLDocument || this[0] instanceof HTMLElement));
}

/**
 * Retrieves a response from the specified web template URL.
 * @param {string} webTemplateURL - The URL of the web template.
 * @returns {object} - The response object retrieved from the web template URL.
 */
function nel_getResponse(webTemplateURL) {
    var response = null;
    try {
        $.ajax({
            type: "GET",
            url: webTemplateURL,
            dataType: "json",
            async: false
        }).done(function (json) {
            response = json;
        });
    } catch (e) {
        console.log("getResponse :: " + e.message);
    }
    return response;
}

/**
 * Replaces a lookup field with a dropdown field.
 * 
 * @param {string} fieldName - The ID of the lookup field.
 * @param {string} entityLogicalName - The logical name of the entity.
 * @param {boolean} [blankDefault=true] - Indicates whether to include a blank option as the default value.
 * @returns {jQuery} - The jQuery object representing the dropdown field.
 */
function nel_replaceLookupWithDropdown(fieldName, entityLogicalName, blankDefault = true) {
    if ($("#" + fieldName).exists()) {
        if ($("#" + fieldName + "_dropdown").exists()) {
            return $("#" + fieldName + "_dropdown");
        }
        $("#" + fieldName).parent().parent().hide();

        let c = $("#" + fieldName).parent().parent().parent();

        $("<select/>", {
            "class": "lookup form-control",
            "id": fieldName + "_dropdown",
        }).appendTo(c);
        // $("#" + fieldName + "_dropdown").append($("<option disabled selected value></option>"))
        $("#" + fieldName + "_dropdown").append($("<option selected value></option>"))
        $("#" + fieldName + "_dropdown").change(function () {
            var id = $("#" + fieldName + "_dropdown").val()
            $("#" + fieldName).attr("value", id);
            $("#" + fieldName + "_entityname").attr("value", entityLogicalName);

        });

        return $("#" + fieldName + "_dropdown");
    }
}

/**
 * Replaces a lookup field with dummy text.
 * @param {string} fieldName - The name of the lookup field.
 * @returns {jQuery} - The jQuery object representing the replaced field.
 */
function nel_replaceLookupWithDummyText(fieldName) {
    if ($("#" + fieldName).exists()) {
        if ($("#" + fieldName + "_txt").exists()) {
            return $("#" + fieldName + "_txt");
        }
        $("#" + fieldName).parent().parent().hide();

        let c = $("#" + fieldName).parent().parent().parent();

        $("<input/>", {
            "class": "form-control",
            "id": fieldName + "_txt",
        }).appendTo(c);
        

        return $("#" + fieldName + "_txt");
    }
}


/**
 * Replaces a lookup field with an input field and a datalist.
 * @param {string} fieldName - The name of the field.
 * @param {string} entityLogicalName - The logical name of the entity.
 * @param {Array} arrayOfOptions - An array of options for the datalist.
 * @returns {jQuery} - The jQuery object representing the replaced input field.
 */
function nel_replaceLookupWithInputList(fieldName, entityLogicalName, arrayOfOptions) {
    if ($("#" + fieldName).exists()) {
        if ($("#" + fieldName + "_dropdown").exists()) {
            return $("#" + fieldName + "_dropdown");
        }
        $("#" + fieldName).parent().parent().hide();

        let c = $("#" + fieldName).parent().parent().parent();

        $("<input />", {
            "class": "lookup form-control",
            "list": fieldName + "_datalist",
            "id": fieldName + "_dropdown",
        }).appendTo(c);
        let dataList = $("<datalist />", { "id": fieldName + "_datalist" });
        dataList.appendTo(c);
        $.each(arrayOfOptions, function (i, item) {
            dataList.append($('<option>', {
                "value": item.Name,
                "data-customvalue": item.Id,
                "data-customvaluetwo": item.IsCampaign,
                "data-customvaluethree": item.CampaignId
            }));
        });
        $("#" + fieldName + "_dropdown").change(function () {
            var value = $("#" + fieldName + "_dropdown").val();
            var id = $("#" + fieldName + "_datalist" + " [value='" + value + "']").data("customvalue");
            $("#" + fieldName).attr("value", id);
            $("#" + fieldName + "_entityname").attr("value", entityLogicalName);

        });

        return $("#" + fieldName + "_dropdown");
    }
}

/**
 * Creates a new validator object for a given field.
 * 
 * @param {string} fieldName - The ID of the field to validate.
 * @param {string} errorMessage - The error message to display if validation fails.
 * @param {string} type - The type of validator.
 * @param {function} evalFunc - The evaluation function used for validation.
 */
function nel_createValidator(fieldName, errorMessage, type, evalFunc) {
    // Create new validator object
    if ($("#" + fieldName).exists()) {
        var Requiredvalidator = document.createElement("span");
        Requiredvalidator.style.display = "none";
        Requiredvalidator.id = type + "Validator" + fieldName;
        Requiredvalidator.controltovalidate = fieldName;
        Requiredvalidator.errormessage = errorMessage
        Requiredvalidator.initialvalue = "";
        Requiredvalidator.evaluationfunction = evalFunc;
        Page_Validators.push(Requiredvalidator);
    }
}

/**
 * Sets a field as required and creates a validator for it.
 * @param {string} fieldName - The ID of the field to be set as required.
 */
function nel_setFieldAsRequired(fieldName) {
    if ($("#" + fieldName).exists()) {
        if (typeof (Page_Validators) == "undefined") {
            return;
        }
        if ($("#" + fieldName) != undefined && $("#" + fieldName + "_label") != undefined) {
            $("#" + fieldName).prop("required", true);
            $("#" + fieldName).closest(".control").prev().addClass("required");

            // Create new validator componentes
            let errorMessage = "<a href='#" + fieldName + "_label'>" + $("#" + fieldName + "_label").html() + " er et obligatorisk felt.</a>";
            let evalFunc = function () {
                var fieldControl = $("#" + fieldName);
                if (fieldControl.is("span")) {
                    var value0 = $("#" + fieldName + "_0").prop("checked");
                    var value1 = $("#" + fieldName + "_1").prop("checked");
                    if (value0 == false && value1 == false) {
                        return false;
                    } else {
                        return true;
                    }
                } else {
                    var value = $("#" + fieldName).val();
                    if (value == null || value == "") {
                        return false;
                    } else {
                        return true;
                    }
                }
            };
            // Remove old validator and add the new validator to the page validators array
            for (i = 0; i < Page_Validators.length; i++) {
                if (Page_Validators[i].id == "RequiredFieldValidator" + fieldName) {
                    // Remove the current field’s ‘Required’ validator from ‘Page_Validators’ array
                    Page_Validators.splice(i,1);
                }
            }
            nel_createValidator(fieldName, errorMessage, "RequiredField", evalFunc);
        }
    }
}

/**
 * Sets a field as non-required.
 * @param {string} fieldName - The name of the field to be set as non-required.
 */
function nel_setFieldAsNonRequired(fieldName) {
    if ($("#" + fieldName).exists()) {
        if (typeof (Page_Validators) == "undefined") {
            return;
        }
        if ($("#" + fieldName) != undefined) {
            $("#" + fieldName).closest(".control").prev().removeClass("required");
            $("#" + fieldName).prop("required", false);

            for (i = 0; i < Page_Validators.length; i++) {
                if (Page_Validators[i].id == "RequiredFieldValidator" + fieldName) {
                    // Remove the current field’s ‘Required’ validator from ‘Page_Validators’ array
                    Page_Validators.splice(i,1);
                }
            }
        }
    }
}

/**
 * Adds a regular expression validator to a field.
 * 
 * @param {string} fieldName - The ID of the field to validate.
 * @param {RegExp} regexp - The regular expression to match against the field's value.
 * @param {string} [errormessage="indeholder ikke godkendte tegn."] - The error message to display if the field's value does not match the regular expression.
 */
function nel_addRegexValidator(fieldName, regexp, errormessage = "indeholder ikke godkendte tegn.") {
    if ($("#" + fieldName).exists()) {
        if (typeof (Page_Validators) == 'undefined') {
            return;
        }

        // Create new validator componentes
        let errorMessage = "<a href='#" + fieldName + "_label'>" + $("#" + fieldName + "_label").html() + " " + errormessage + "</a>";
        let evalFunc = function () {
            return regexp.test($("#" + fieldName).val());
        };

        for (i = 0; i < Page_Validators.length; i++) {
            if (Page_Validators[i].id == "RegExValidator" + fieldName) {
                // Remove the current field’s ‘Required’ validator from ‘Page_Validators’ array
                Page_Validators.splice(i,1);
            }
        }

        // Create and add the new validator to the page validators array
        nel_createValidator(fieldName, errorMessage, "RegEx", evalFunc);
    }
}


/**
 * Adds a less than validator for two fields.
 * @param {string} fieldName1 - The ID of the first field.
 * @param {string} fieldName2 - The ID of the second field.
 * @param {boolean} [overruleError=false] - Indicates whether to override the default error message.
 * @param {string} [errormessage=" kan ikke være større end "] - The error message to display when the validation fails.
 */
function nel_addLessThanValidator(fieldName1, fieldName2, overruleError = false, errormessage = " kan ikke være større end ") {
    if ($("#" + fieldName1).exists() && $("#" + fieldName2).exists()) {
        if (typeof (Page_Validators) == 'undefined') {
            return;
        }
        let errorMessage = "";
        // Create new validator componentes
        if (overruleError === false) {
            errorMessage = "<a href='#" + fieldName2 + "_label'>" + $("#" + fieldName2 + "_label").html() + " " + errormessage + "<a href='#" + fieldName1 + "_label'>" + $("#" + fieldName1 + "_label").html() + "</a>";
        } else {
            errorMessage = errorMessage
        }
        let evalFunc = function () {
            //ignore if both values are empty or equal
            if ($("#" + fieldName1).val() === '' && $("#" + fieldName2).val() === '' || $("#" + fieldName1).val() === $("#" + fieldName2).val()) {
                return true;
            }
            return ($("#" + fieldName1).nval() ?? 0) < ($("#" + fieldName2).nval() ?? 0 );
        };
        // Create and add the new validator to the page validators array
        nel_createValidator(fieldName1, errorMessage, "LessThan", evalFunc);
    }
}

/**
 * Adds an equal validator for two field names.
 * @param {string} fieldName1 - The first field name.
 * @param {string} fieldName2 - The second field name.
 * @param {boolean} [overruleError=false] - Flag indicating whether to override the default error message.
 * @param {string} [errormessage=" kan ikke være enes med "] - The error message to display when the fields are not equal.
 */
function nel_addEqualValidator(fieldName1, fieldName2, overruleError = false, errormessage = " kan ikke være enes med ") {
    if ($("#" + fieldName1).exists() && $("#" + fieldName2).exists()) {
        if (typeof (Page_Validators) == 'undefined') {
            return;
        }
        let errorMessage = ""
        // Create new validator componentes
        if (overruleError === false) {
            errorMessage = "<a href='#" + fieldName1 + "_label'>" + $("#" + fieldName1 + "_label").html() + " " + errormessage + "<a href='#" + fieldName2 + "_label'>" + $("#" + fieldName2 + "_label").html() + "</a>";
        } else {
            errorMessage = errormessage
        }
        let evalFunc = function () {
            //ignore if both values are empty
            if ($("#" + fieldName1).val() === '' && $("#" + fieldName2).val() === '') {
                return true;
            }
            return ($("#" + fieldName1).val() ?? 0) != ($("#" + fieldName2).val() ?? 0);
        };
        // Create and add the new validator to the page validators array
        nel_createValidator(fieldName1, errorMessage, "Equal", evalFunc);
    }
}

/**
 * Retrieves the URL query parameters and returns them as an array.
 * @param {boolean} [isChild=false] - Indicates whether the function should retrieve query parameters from the parent window's URL.
 * @returns {Array} - An array containing the query parameters.
 */
function nel_getUrlVars(isChild = false) {
    var vars = [], hash;
    var hashes = null;
    if (isChild === true) {
        hashes = parent.window.location.href.slice(parent.window.location.href.indexOf('?') + 1).split('&');
    } else {
        hashes = window.location.href.slice(window.location.href.indexOf('?') + 1).split('&');
    }
    for (var i = 0; i < hashes.length; i++) {
        hash = hashes[i].split('=');
        vars.push(hash[0]);
        vars[hash[0]] = hash[1];
    }
    return vars;
}


/**
 * Sets the maximum date for a date picker field.
 * 
 * @param {string} fieldName - The ID of the date picker field.
 * @param {Date} maxDate - The maximum date allowed.
 */
function nel_datePickerSetMaxDate(fieldName, maxDate) {
    // Ensure the field exists
    if ($("#" + fieldName).length) {
        console.log("Field found: ", fieldName);
        
        var $datePicker = $("#" + fieldName).next().data("DateTimePicker");
        console.log("DatePicker instance for", fieldName, $datePicker);
        
        if (!$datePicker) {
            console.error("DateTimePicker not found for field:", fieldName);
            return;
        }

        $datePicker.maxDate(maxDate);

        // Define a named function for the dp.change event handler
        function handleDatePickerChange_MaxDate(e) {
            var curDate = moment($("#" + fieldName).val());
            var clonedMaxDate = maxDate.clone().startOf('day'); // Clone and clear time portion for correct results
            
            if (curDate > clonedMaxDate) {
                alert("Ugyldig dato");
                $('input[aria-describedby="' + fieldName + '_description"]').val(null); // Clear the field value
            }
        }

        // Remove any existing dp.change event with the custom namespace .maxDateChange
        $("#" + fieldName).next().off("dp.change.maxDateChange");
        
        // Attach the new dp.change event handler
        $("#" + fieldName).next().on("dp.change.maxDateChange", handleDatePickerChange_MaxDate);
    } else {
        console.error("Field not found:", fieldName);
    }
}
function nel_datePickerSetMinDate(fieldName, minDate) {
    if ($("#" + fieldName).length > 0) {
        var $datePicker = $("#" + fieldName).next().data("DateTimePicker");

        if (!$datePicker) {
            console.error("DateTimePicker instance not found for field: " + fieldName);
            return;
        }

        // Get the current maxDate set in the date picker
        var currentMaxDate = $datePicker.maxDate();

        // Check if minDate is after maxDate
        if (minDate.isAfter(currentMaxDate)) {
            console.warn("minDate is after maxDate. Adjusting minDate to maxDate to prevent error.");
            minDate = currentMaxDate.clone();  // Adjust minDate to match maxDate
        }


        $datePicker.minDate(minDate);

        function handleDatePickerChange_MinDate(e) {
            var curDate = moment($("#" + fieldName).val());

            var clonedMinDate = minDate.clone().startOf('day');

            if (curDate.isBefore(clonedMinDate)) {
                alert("Ugyldig dato");
                $('input[aria-describedby="' + fieldName + '_description"]').val(null);  // Clear the field value
            }
        }

        $("#" + fieldName).next().off("dp.change.minDateEvent");
        $("#" + fieldName).next().on("dp.change.minDateEvent", handleDatePickerChange_MinDate);

    } else {
        console.error("Field not found: " + fieldName);
    }
}


/**
 * Adds a date picker event listener to the specified element.
 * @function nel_AddDatePickerEvent
 * @memberof window
 * @returns {void}
 */
function nel_AddDatePickerEvent(){
    $('#liquid_form .datetimepicker').on('dp.change', function (ev) {
        const relatedField = ev.target.previousElementSibling;
        if (relatedField) {
            const myEvent = new CustomEvent('datetimepicker-changed', {
                bubbles: true,
                detail: {
                    logicalName: () => relatedField.id
                }
            });
            this.dispatchEvent(myEvent);
        }
    })
}

/**
 * Adds a change event listener to a date picker field.
 * 
 * @param {string} fieldName - The ID of the date picker field.
 * @param {function} changeFunc - The function to be executed when the change event occurs.
 */
function nel_datePickerAddChange(fieldName, changeFunc) {
    if ($("#" + fieldName).exists()) {
        $("#" + fieldName).next().on("dp.change", changeFunc);
    }
}

/**
 * Executes a flow by sending a POST request to the specified URL with the provided object as the request body.
 * @param {string} url - The URL to send the POST request to.
 * @param {object} obj - The object to be sent as the request body.
 * @returns {Promise<Response>} - A promise that resolves to the response of the POST request.
 */
async function nel_runFlow(url, obj) {
    const response = await fetch(url, {
        method: 'POST',
        mode: 'cors',
        cache: 'no-cache',
        credentials: 'same-origin',
        headers: {
            'Content-Type': 'application/json'
        },
        redirect: 'follow',
        referrerPolicy: 'no-referrer',
        body: JSON.stringify(obj)
    });
    return response;
}

/**
 * Represents the box types for Nel.
 * @typedef {Object} nel_boxType
 * @property {string} warning - The CSS class for a warning box.
 * @property {string} error - The CSS class for an error box.
 * @property {string} info - The CSS class for an info box.
 * @property {string} success - The CSS class for a success box.
 */
let nel_boxType = {
    warning: "alert alert-warning",
    error: "alert alert-danger",
    info: "alert alert-info",
    success: "alert alert-success"

}

/**
 * Creates a box element with specified properties.
 * @param {string} typeOfBox - The type of box to create.
 * @param {string} [divID="DangerDiv"] - The ID of the box element.
 * @param {string} [divTitle="Fejl"] - The title of the box element.
 * @param {jQuery} [createAfter=$(".page-header")] - The element after which the box should be created.
 * @returns {Object} - An object with properties and methods related to the created box element.
 */
function nel_CreateBox(typeOfBox, divID = "DangerDiv", divTitle = "Fejl", createAfter = $(".page-header")) {
    if ($("#" + divID).length > 0) {
        $("#" + divID).remove();
    }
    createAfter.after('<div id="' + divID + '" class="' + typeOfBox + '"><h3>' + divTitle + '</h3></div>');
    $("#" + divID).append("<ul id='" + divID + "ul'></>");
    return warningBoxObj = {
        div: $("#" + divID),
        lst: $("#" + divID + "ul"),
        addLi: (txt, liID) => {
            warningBoxObj.lst.append("<li id='" + liID + "'>" + txt + "</>");
        },
        removeLi: liID => {
            $("#" + liID).remove();
        },
        clearLst: () => {
            warningBoxObj.lst.empty();
        },
        remove: () => {
            warningBoxObj.divObj.remove();
        }

    }
}

/**
 * Executes a workflow on a target entity record.
 *
 * @param {string} workflowId - The GUID of the workflow to execute.
 * @param {string} entityLogicalName - The logical name of the target entity.
 * @param {string} recordId - The GUID of the target record.
 * @param {function} successFunc - The success callback function.
 * @param {function} errorFunc - The error callback function.
 */
function nel_ExecuteWorkflow(workflowId, entityLogicalName, recordId, successFunc, errorFunc) {
    var urlService = "/_services/execute-workflow/e78aead7-42db-ec11-bb3d-000d3a213e08"; // execute workflow URL
    var params = {};
    var workflow = {};
    workflow.LogicalName = "workflow";
    workflow.Id = workflowId; // guid of the workflow to execute
    var data = {};
    data.LogicalName = entityLogicalName; // logical name of the target entity
    data.Id = recordId; // guid of the target record
    params.workflow = workflow;
    params.entity = data;
    var jData = JSON.stringify(params);
    shell.ajaxSafePost({
        type: "POST",
        contentType: "application/json",
        url: urlService,
        data: jData
    }).done(function (response) {
        successFunc(response);
    }).fail(function (response) {
        errorFunc(response);
    });
}

/**
 * Bypasses the cache and updates a field value in the specified schema.
 * @param {string} id - The ID of the record.
 * @param {string} schemaName - The name of the schema.
 * @param {string} field - The name of the field to update.
 */
function nel_ByPassCache (id, schemaName, field){
    try {
        var n = Date.now();
        var timestamp = n.toString();
        webapi.safeAjax({
          type: "PUT",
          url: "/_api/" + schemaName + "(" + id + ")/"+ field,
          contentType: "application/json",
          data: JSON.stringify({
            "value": timestamp
          }),
          success: function(res) {
            console.log(res);
          }
        });
      } catch (err) {
        alert(err.message);
      }
}

/**
 * Searches for data in a table based on a query parameter from the URL.
 * @function nel_SearchFromQuery
 */
function nel_SearchFromQuery(){
    var maxAttempts = 2500; // Maximum number of attempts to wait for the table data
    var attempts = 0; // Counter for attempts

    // Check if the table exists and has any data, if not, poll every 500 milliseconds
    var tableCheckInterval = setInterval(function () {
        attempts++; // Increment the attempts counter
        var table = $("table.table.table-striped tbody tr");
        if (table.length > 0 || attempts >= maxAttempts) {
            // Table has data or maximum attempts reached, execute your code here
            clearInterval(tableCheckInterval); // Stop polling
            if (table.length > 0) {
                console.log("Table with data is ready!");
                // Get the value of the 'search' parameter from the URL
                var searchValue = nel_getUrlVars()['search'];

                // Set the search field value
                $('.query.form-control').val(searchValue);

                // Trigger a click event on the search button
                $('.btn.btn-default.btn-hg').trigger('click');
            } else {
                console.log("Table data not found after maximum attempts.");
            }
        }
    }, 500); // Poll every 500 milliseconds

}